@@ -1,7 +1,7 @@
 import os, sys
 
 import subprocess
-import signal
+import signal, platform
 from puke.Console import *
 from puke.Std import *
 
@@ -15,8 +15,12 @@ def alarm_handler(signum, frame):
 
 
 def sh (command, header = None, output = True, timeout = None, std = None):
+
+    isWouinWouin = (platform.system() == 'Windows')
+
     if isinstance(command, list):
-        command = " ; ".join(command)
+        command = " \n ".join(command)
+    
 
 
 
@@ -33,23 +37,36 @@ def sh (command, header = None, output = True, timeout = None, std = None):
     
     result = ""
 
-    if timeout:
+    if timeout and not isWouinWouin:
         signal.signal(signal.SIGALRM, alarm_handler)
         signal.alarm(timeout)  # 5 seconds
     
 
     try:
+        executable = '/bin/bash'
         command = "export PYTHONIOENCODING=utf-8;" + command
-        cProcess = subprocess.Popen(command, stdout = subprocess.PIPE, shell = True, stderr= subprocess.PIPE, executable='/bin/bash', env=os.environ)
+
+
+        if isWouinWouin:
+            executable = None
+            command = "bash.exe -c '" + command.replace("'", "\\'") + "'"
+        
+
+        cProcess = subprocess.Popen(command, stdout = subprocess.PIPE, shell = True, stderr= subprocess.PIPE, executable=executable, env=os.environ)
         (rStdout, rStderr) = cProcess.communicate()
-        signal.alarm(0)
+        code = cProcess.returncode
+
+        if not isWouinWouin:
+            signal.alarm(0)
+
     except Alarm:
         console.debug('taking too long (%s)' % command)
         cProcess.kill()
         rStdout = rStderr = ''
+        code = 0
     
     if std and isinstance(std, Std):
-        std.set(rStdout, rStderr)
+        std.set(rStdout, rStderr, code)
 
     rStdout = "%s\n%s\n%s" % (rStdout if rStdout else '', stdErrMarker if rStderr else '', rStderr if rStderr else '')
 
